/*
    Copyright (c) 2009 digitalSTROM.org, Zurich, Switzerland
    Copyright (c) 2009 futureLAB AG, Winterthur, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DSS_H_INCLUDED
#define DSS_H_INCLUDED

#ifdef HAVE_CONFIG_H
  #include "config.h"
#endif

#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/shared_ptr.hpp>

#include <vector>

namespace dss {

  class DSS;
  class Subsystem;
  class WebServer;
  class DS485Interface;
  class PropertySystem;
  class Metering;
  class FakeMeter;
  class EventRunner;
  class EventQueue;
  class EventInterpreter;
  class Apartment;
  class WebServices;
#ifdef WITH_SIM
  class DSSim;
#endif

  typedef enum {
    ssInvalid,
    ssCreatingSubsystems,
    ssLoadingConfig,
    ssInitializingSubsystems,
    ssStarting,
    ssRunning,
    ssTerminating
  } aDSSState;

  /** Main class
    *
    */
  class DSS {
  private:
    static DSS* m_Instance;
    std::vector<Subsystem*> m_Subsystems;
    time_t m_TimeStarted;
    boost::shared_ptr<WebServer> m_pWebServer;
    boost::shared_ptr<DS485Interface> m_pDS485Interface;
    boost::shared_ptr<PropertySystem> m_pPropertySystem;
    boost::shared_ptr<Apartment> m_pApartment;
#ifdef WITH_SIM
    boost::shared_ptr<DSSim> m_pSimulation;
#endif
    boost::shared_ptr<EventRunner> m_pEventRunner;
    boost::shared_ptr<WebServices> m_pWebServices;
    boost::shared_ptr<EventInterpreter> m_pEventInterpreter;
    boost::shared_ptr<EventQueue> m_pEventQueue;
    boost::shared_ptr<Metering> m_pMetering;
    boost::shared_ptr<FakeMeter> m_pFakeMeter;
    std::string m_DataDirectory;

    aDSSState m_State;

    /// Private constructor for singleton
    DSS();

    bool loadConfig();
    void addDefaultInterpreterPlugins();

    int getUptime() const;
  public:
    ~DSS();
    bool initialize(const std::vector<std::string>& _properties);
    void run();

    static DSS* getInstance();
    static bool hasInstance();
    static void shutdown();
#ifdef WITH_TESTS
    static void teardown();
#endif
    static std::string versionString();

#ifndef WIN32
    static void handleSignal(int signum);
#endif

    aDSSState getState() const { return m_State; }

    DS485Interface& getDS485Interface() { return *m_pDS485Interface; }
    Apartment& getApartment() { return *m_pApartment; }
#ifdef WITH_SIM
    DSSim& getSimulation() { return *m_pSimulation; }
#endif
    EventRunner& getEventRunner() { return *m_pEventRunner; }
    WebServices& getWebServices() { return *m_pWebServices; }
    EventQueue& getEventQueue() { return *m_pEventQueue; }
    Metering& getMetering() { return *m_pMetering; }
    PropertySystem& getPropertySystem() { return *m_pPropertySystem; }
    WebServer& getWebServer() { return *m_pWebServer; }
    EventInterpreter& getEventInterpreter() { return *m_pEventInterpreter; }

    const std::string& getDataDirectory() const { return m_DataDirectory; }
    void setDataDirectory(const std::string& _value);
  }; // DSS

}

#endif // DSS_H_INCLUDED

